jQuery.cachedScript = function(url, options) {
  options = $.extend(options || {}, {
    dataType: "script",
    cache: true,
    url: url
  });
  return jQuery.ajax(options);
};

/**
 * Controla funções que utilizam mapas e GPS
 * @type {Object}
 */
var gps = {

  initialize: function() {
    $('.user-local').html("Iniciando aplicativo...");
    $.cachedScript("http://maps.googleapis.com/maps/api/js?key=AIzaSyBv9ZKR7s3jRIaDyK2DG1ZT26Id2HMZcvw&sensor=true&v=3.exp&callback=gps.setMapIni").done(function(script, textStatus) {});
  },

  /**
   * Busca localização para carregar o mapa pela primeira vez que o aplicativo
   * é aberto
   */
  setMapIni: function() {
    $('.user-local').html("Obtendo sua localização");
    navigator.geolocation.getCurrentPosition(gps.setMap, gps.onError, {
      enableHighAccuracy: true,
      maximumAge: 5000
    });
  },

  /**
   * @param {position} - cordova-plugin-geolocation
   */
  setMap: function(position) {
    $('.user-local').html("Carregando o mapa");
    if (!localStorage.hasMototaxiDialog) {
      $.dialog.info(
        "Olá!",
        "Estamos carregando o mapa e obtendo o seu endereço. Por motivo de segurança não exibiremos os mototaxistas no mapa."
      );
      localStorage.hasMototaxiDialog = true;
    }
    lLatitude = aLatitude;
    lLongitude = aLongitude;
    aLatitude = position.coords.latitude;
    aLongitude = position.coords.longitude;
    var latLong = new google.maps.LatLng(aLatitude, aLongitude);
    var mapOptions = {
      center: latLong,
      zoom: 16,
      disableDefaultUI: true,
      mapTypeId: google.maps.MapTypeId.ROADMAP
    };
    map = new google.maps.Map(document.getElementById("map"), mapOptions);
    // Inicializa a variável de push notifications
    pushNotification = window.plugins.pushNotification;
    // Recebe regid e executa user.ping para o servidor
    gcm.initialize();
    resizeMap();
    // Aprimora localização
    gps.setWatch();
    gps.updateAddress();
    $(gps.centerLocationButton);
  },

  /**
   * @param  {object} error - Objeto que contém informações do erro
   * @return {null}
   */
  onError: function(error) {
    console.log(JSON.stringify(error));
    gps.setMapIni();
    /*
    // alert('code: ' + error.code + '\n' + 'message: ' + error.message + '\n');
    // Posiciona o mapa no centro de Petrolina
    var latLong = new google.maps.LatLng(-9.396241692697288, -40.502262169836385);
    var mapOptions = {
      center: latLong,
      zoom: 16,
      disableDefaultUI: true,
      mapTypeId: google.maps.MapTypeId.ROADMAP
    };
    map = new google.maps.Map(document.getElementById("map"), mapOptions);
    // Inicializa a variável de push notifications
    pushNotification = window.plugins.pushNotification;
    // Recebe regid e executa user.ping para o servidor
    gcm.initialize();
    */
  },

  /**
   * Inicia escuta no GPS
   */
  setWatch: function() {
    watchID = navigator.geolocation.watchPosition(gps.setPosition, gps.onError, {
      enableHighAccuracy: true,
      maximumAge: 5000
    });
  },

  /**
   * Centraliza mapa com posição aprimorada
   * Obtém endereço do centro do mapa
   * @param {position} - cordova-plugin-geolocation
   */
  setPosition: function(position) {
    if (position.coords.accuracy < 100) {
      var longitude = position.coords.longitude;
      var latitude = position.coords.latitude;
      var latLong = new google.maps.LatLng(latitude, longitude);
      lLongitude = aLongitude;
      lLatitude = aLatitude;
      aLongitude = longitude;
      aLatitude = latitude;
      map.setCenter(latLong);
      navigator.geolocation.clearWatch(watchID);
    } else {
      $('.user-local').html("Aprimorando sua localização (" + position.coords.accuracy + ")");
    }
  },

  /**
   * Cria rota entre o ponto atual do passageiro e o ponto do mototaxista
   * @param {float} latitude
   * @param {float} longitude
   */
  setRoute: function(latitude, longitude) {
    directionsService = new google.maps.DirectionsService();
    var rendererOptions = {
      suppressMarkers: true,
      suppressPolylines: true
    };
    directionsDisplay = new google.maps.DirectionsRenderer(rendererOptions);
    directionsDisplay.setMap(map);
    var start = new google.maps.LatLng(aLatitude, aLongitude);
    var end = new google.maps.LatLng(latitude, longitude);
    var request = {
      origin: start,
      destination: end,
      travelMode: google.maps.TravelMode.DRIVING
    };
    directionsService.route(request, function(result, status) {
      if (status == google.maps.DirectionsStatus.OK) {
        directionsDisplay.setDirections(result);
        marker_client = new google.maps.Marker({
          position: start,
          map: map,
          icon: "img/marker-client.svg"
        });
        marker_worker = new google.maps.Marker({
          position: end,
          map: map,
          icon: "img/marker-moto.svg"
        });
      } else {
        $.alert('Problemas com as rotas: ' + status);
      }
    });
  },

  /**
   * Permite obtenção de endereço do ponto central do mapa
   * quando o mesmo é completamente carregado ou quando o mapa é arrastado
   * @return {null}
   */
  updateAddress: function() {
    // Carrega endereço quando carregar todo o mapa
    tileEvent = google.maps.event.addListener(map, 'tilesloaded', gps.geocoder);
    dragEvent = google.maps.event.addListener(map, 'dragend', gps.geocoder);
  },

  disableUpdateAddress: function() {
    google.maps.event.removeListener(tileEvent);
    google.maps.event.removeListener(dragEvent);
  },

  geocoder: function() {
    $("#btn-action").attr("ready", "false");
    $('.user-local').html("Obtendo endereço");
    var local = map.getCenter();
    aLatitude = local.lat();
    aLongitude = local.lng();
    var geocoder = new google.maps.Geocoder();
    geocoder.geocode({
      'location': local
    }, function(results, status) {
      if (status == google.maps.GeocoderStatus.OK) {
        user.setPlaceId(results[results.length - 3]["place_id"]);
        $("#btn-action").attr("ready", "true");
        getMotoOnline();
        if (results[0]) {
          $('.user-local').html(results[0].formatted_address);
        } else if (results[1]) {
          $('.user-local').html(results[1].formatted_address);
        } else {
          $('.user-local').html('Localização...');
        }
      } else {
        console.log(JSON.stringify(google.maps.GeocoderStatus));
        gps.geocoder();
      }
    });
  },

  /**
   * Permite que a localização seja reajustada para a localização inicial
   * ao clicar no botão de ajustar posição
   * @return {null}
   */
  centerLocationButton: function() {
    $(".updateGps").on("tapstart", function() {
      var latLong = new google.maps.LatLng(lLatitude, lLongitude);
      map.setCenter(latLong);
    });
  },

  /**
   * Calcula a distância entre dois pontos
   * @param  {float} - Latitude do primeiro ponto
   * @param  {float} - Longitude do primeiro ponto
   * @param  {float} - Latitude do segundo ponto
   * @param  {float} - Longitude do segundo ponto
   * @return {float} - Distância em metros entre os dois pontos
   */
  getDistance: function(lat1, lon1, lat2, lon2) {
    var raio = 6378137; // Raio da terra em metros
    var dLatitude = gps.deg2rad(lat2 - lat1); // deg2rad
    var dLongitude = gps.deg2rad(lon2 - lon1);
    var calculo = Math.sin(dLatitude / 2) * Math.sin(dLatitude / 2) +
      Math.cos(gps.deg2rad(lat1)) * Math.cos(gps.deg2rad(lat2)) *
      Math.sin(dLongitude / 2) * Math.sin(dLongitude / 2);
    var resultado = 2 * Math.atan2(Math.sqrt(calculo), Math.sqrt(1 - calculo));
    var distancia = raio * resultado;
    return distancia;
  },

  /**
   * Método para conversão de radianos
   * @param  {int}
   * @return {float} - Valor convertido
   */
  deg2rad: function(deg) {
    return deg * (Math.PI / 180);
  }

};
